/*
    Method for calculating length of optimal paths
    Input variables:
        - char dir[]:   File directory
        - int Ngrid:    Dimension of grid (NxN)
        - int mi:       Which "minimumvalues_.dat to be read
        - int startx:   X value for one of the path ends
        - int starty:   Y value for the same path end

    Return variable:
        - double:   totdist = Total length of input path

    Other output:
        - FILE "sorted_minpath_.dat  containing the sorted list of points in optimal path
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>


double calc_pathlength(char directory[], int Ngrid, int mi) {

    printf("\n Calculating pathlength \n");

    char dir[150];

    sprintf(dir, directory);

//------------------------------------------------------------------------------------------------------
    // Create files in given directory

    char nPo[200], unsn[200], sorn[200];

    sprintf(nPo, "%s%s", dir, "\\number_val.dat");
    sprintf(unsn, "%s%s%i%s", dir, "\\minimumvalues", mi, ".dat");
    sprintf(sorn, "%s%s%i%s", dir, "\\sorted_minpath", mi, ".dat");

    FILE *nPoints = fopen(nPo, "r");   // Number of points in optimal path
    FILE *unspath = fopen(unsn, "r");  // Unsorted list of points in optimal path
    FILE *sorpath = fopen(sorn, "w");  // Sorted list of points in optimal path

//-----------------------------------------------------------------------------------------------------


    int N;    // pathlength
    fscanf(nPoints, "%i", &N);
    fclose(nPoints);


    int i, j, tx, ty, icount;
    int x[N], y[N], neighbor[N];
    double e, dist, totdist = 0.0;

    for (i = 0; i < N; i++) {
        fscanf(unspath, "%i %i %lf\n", &tx, &ty, &e);
        x[i] = tx;
        y[i] = ty;
    }
    fclose(unspath);


    // Localize the endpoint by finding the number of neighbors for each point
    // The ends will be given by having 1 neighbor, while the remaining have 2.

    for (i = 0; i < N; i++) {
        icount = 0;
        for(j = 0; j < N; j++) {
            if(j != i){
                dist = sqrtf(  (x[i] - x[j])*(x[i] - x[j]) + (y[i] - y[j])*(y[i] - y[j]) );
                if(dist <= (sqrtf(2.0) + 0.1))
                    icount++;
                if(icount == 2)
                    j = N;
            }
        }
        neighbor[i] = icount;
    }

    // Search for the first neigbor
    i = 0;
    while(neighbor[i] != 1){
        i++;
    }

    tx = x[i] - 1;
    ty = y[i] - 1; // Initialization

    printf("Start point: %i %i", x[i], y[i]);

    for (i = 0; i < N; i++) {

        for (j = 0; j < N; j++) {
            dist = sqrtf( (x[j] - tx)*(x[j] - tx) + (y[j] - ty)*(y[j] - ty) );

            if( (dist >= 0.99 && dist <= 1.01) || ((dist >= sqrtf(2.0) - 0.1) && (dist  <= sqrtf(2.0) + 0.1))) {

                tx = x[j];
                ty = y[j];

                // prevent backlooping by mooving already processed points out of area
                x[j] = Ngrid + 2;
                y[j] = Ngrid + 2;

                j = N;  // Break the inner loop
            }
        }

        totdist += dist;

        fprintf(sorpath, "%i %i %lf\n", tx, ty, e);
    }
    fclose(sorpath);

    printf("total distance %lf\n", totdist);

    return totdist;
}
