/*
Dette er en enkel versjon av grid.c.
Programmet skal hente inn data, transformere, og kalle pathscape.c fra denne mappen.
Det aktulle pathcsape.c skal kunne kjres flere ganger med ny verdi av alfa.
Minimumsruten skal lagres for hver alfa slik at de ulike verdiene kan sammenliknes i et plot.
*/


#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "pathscapeorg.c"
#include "replot.c"
#include "calclength.c"

double *x, *y, *xt, *yt;
double **grid2d;


int main () {

    /*
    ** Parameters
    **  N       :   Dimension of grid N*N
    **  length  :   number of datapoints ( - 1)
    **  R       :   Earths radius [m]
    **  pi      :   pi
    */

    int N;                  // The dimension of the grid is determined later in the program
    int length = 1405712;
    int R = 6371000;
    double pi = 3.14159265359;
    double alpha;
    int times = 21; // Number of different realizations of alpha
    double delta = 0.1;    // Interval between alpha values

    /*
    **  Directory and filenames
    **  Check that the inputfile name "datan" is correct!
    */
    char dir[150] = "c:\\users\\mads\\documents\\ntnu\\masteroppgave\\resultater\\25.03.2014\\diagonal_1";
    char datan[200], out1n[200], out2n[200], out3n[200], out4n[200], out6n[200], out7n[200];
    char gpl1n[200], gpl2n[200], gpl3n[200], gpl4n[200], gpl5n[200], gpl6n[200];

    sprintf(datan, "%s%s",dir,"\\coords_routes_sorted.dat");
    sprintf(out1n, "%s%s",dir,"\\gridvalues.dat");
    sprintf(out2n, "%s%s",dir,"\\gridvalues_reduced.dat");
    sprintf(out3n, "%s%s",dir,"\\coord_trans.dat");
    sprintf(out4n, "%s%s",dir,"\\pathlength_alpha.dat");
    sprintf(out6n, "%s%s",dir,"\\coords_reduced.dat");
    sprintf(out7n, "%s%s",dir,"\\info.dat");
    sprintf(gpl1n, "%s%s",dir,"\\plot_full.txt");
    sprintf(gpl2n, "%s%s",dir,"\\plot_part.txt");
    sprintf(gpl3n, "%s%s",dir,"\\plot_alphas.txt");
    sprintf(gpl4n, "%s%s",dir,"\\plot_alphas_data.txt");
    sprintf(gpl5n, "%s%s",dir,"\\plot_alphaVSlength.txt");
    sprintf(gpl6n, "%s%s",dir,"\\plot_alphaVSlengthNorm.txt");

    //Help-parameters
    int i, j, xint, yint, change, newlength = length, s, drop;
    double xi, yi, longmin = 10000, longmax = 0, latmin = 10000, latmax = 0, xmax, ymax, xmin, ymin, degrad;
    double startlong, startlat, endlong, endlat, longdif, latdif, avlong, avlat, dimension, gridsize, pathlength, firstpath;
    double w1, w2, w3, w4, r1, r2, r3, r4, rtot;
    double dropd;
    char name[1000];
    char gnu[1000];
    char choise = 'n';

    /*
    ** Allocation
    */
    x = (double*) malloc(sizeof(double)*length);
    y = (double*) malloc(sizeof(double)*length);


    // OPENING FILES

    // Input
    FILE *data = fopen(datan,"r");

    // Output
    FILE *out1 = fopen(out1n,"w");
    FILE *out2 = fopen(out2n,"w");
    FILE *out3 = fopen(out3n,"w");
    FILE *out4 = fopen(out4n,"w");
    FILE *out6 = fopen(out6n,"w");
    FILE *out7 = fopen(out7n,"w");

    // Plotscripts
    FILE *gpl1 = fopen(gpl1n,"w");
    FILE *gpl2 = fopen(gpl2n,"w");
    FILE *gpl3 = fopen(gpl3n,"w");
    FILE *gpl4 = fopen(gpl4n,"w");
    FILE *gpl5 = fopen(gpl5n,"w");
    FILE *gpl6 = fopen(gpl6n,"w");




    // Read the dataset
    for (i = 0; i < length; i++) {

        //fscanf(data,"%lf %lf %lf %i %i %i %i %i %i %i %i %[^\n]s", &xi, &yi, &dropd, &drop, &drop, &drop, &drop, &drop, &drop, &drop, &drop, &name);
        fscanf(data, "%lf %lf\n", &xi, &yi);
        x[i] = xi;
        y[i] = yi;

        if (xi >= longmax)
            longmax = xi;
        if (xi <= longmin)
            longmin = xi;
        if (yi >= latmax)
            latmax = yi;
        if (yi <= latmin)
            latmin = yi;
    }
    fclose(data);

    // Create the plotscripts

    // For the original dataset
    fprintf(gpl1, "cd '%s'\n",dir);
    fprintf(gpl1, "set term png size 1024,1024\n");
    fprintf(gpl1, "unset key\n");
    fprintf(gpl1, "set out 'original_data.png'\n");
    fprintf(gpl1, "plot 'coords_routes_sorted.dat' using 1:2\n");
    fprintf(gpl1, "exit");
    fclose(gpl1);


    // For plotting multiple realizations
    fprintf(gpl3, "cd '%s' \n", dir);
    fprintf(gpl3, "set term png enhanced\n");
    fprintf(gpl3, "set key outside\n");
    fprintf(gpl3, "set out 'diagonal.png'\n");
    fprintf(gpl3, "set xlabel \"i\"\n");
    fprintf(gpl3, "set ylabel \"j\"\n");
    fprintf(gpl3, "plot ");
    for(i = 0; i < times; i++) {
        fprintf(gpl3, "'minimumvalues%i.dat' using 1:2 title \"%.2f\" ",i, delta*i);
        if(i < times-1)
            fprintf(gpl3, ", ");
    }
    fprintf(gpl3,"\n");
    fprintf(gpl3,"exit");
    fclose(gpl3);


    // For plotting path length vs alpha
    fprintf(gpl5, "cd '%s' \n", dir);
    fprintf(gpl5, "set term png enhanced\n");
    fprintf(gpl5, "unset key\n");
    fprintf(gpl5, "set encoding utf8\n");
    fprintf(gpl5, "set xlabel \"{/Symbol a}\" \n");
    fprintf(gpl5, "set ylabel \"Length\" \n");
    fprintf(gpl5, "set out 'alphaVSlength.png'\n");
    fprintf(gpl5, "plot 'pathlength_alpha.dat' using 1:2 with points pointtype 7 pointsize 1 lc rgb 'black', ");
    fprintf(gpl5, "'pathlength_alpha.dat' using 1:2 with lines lc rgb 'black'\n");
    fprintf(gpl5, "exit");
    fclose(gpl5);

    // For plotting path length normalized vs alpha
    fprintf(gpl6, "cd '%s' \n", dir);
    fprintf(gpl6, "set term png enhanced\n");
    fprintf(gpl6, "unset key\n");
    fprintf(gpl6, "set encoding utf8\n");
    fprintf(gpl6, "set xlabel \"{/Symbol a}\" \n");
    fprintf(gpl6, "set ylabel \"length / min(length)\" \n");
    fprintf(gpl6, "set out 'alphaVSlengthNorm.png'\n");
    fprintf(gpl6, "plot 'pathlength_alpha.dat' using 1:3 with points pointtype 7 pointsize 1 lc rgb 'black', ");
    fprintf(gpl6, "'pathlength_alpha.dat' using 1:3 with lines lc rgb 'black'\n");
    fprintf(gpl6, "exit");
    fclose(gpl6);

    //Plot the original data:
    sprintf(gnu,"%s %s","gnuplot", gpl1n);
    system(gnu);

    // ***********************************************************************************************
    // Set the endpoints for analysis
    // Make sure it is a square!


    startlong = 105.0;
    endlong =  108.0;
    startlat = 0.0;
    endlat = 3.0;

    /*
    startlong = 0.0;
    endlong =  100.0;
    startlat = 0.0;
    endlat = 100.0;
    */
    /*
    startlong = 3.95;
    endlong = 4.25;
    startlat = 60.0;
    endlat = 60.3;
    */

    fprintf(out7 ,"startlong: %lf\n", startlong);
    fprintf(out7 ,"endlong: %lf\n", endlong);
    fprintf(out7 ,"startlat: %lf\n", startlat);
    fprintf(out7 ,"endlat: %lf\n", endlat);

    longdif = fabs(endlong - startlong);
    latdif = fabs(endlat - startlat);

    if  (longdif > latdif)  {
        avlat = 0.5*(endlat + startlat);
        longmin = fmin(endlong, startlong);
        longmax = fmax(endlong, startlong);
        latmin =  avlat - 0.5*longdif;
        latmax =  avlat + 0.5*longdif;
    }
    else if (longdif < latdif)   {
        avlong = 0.5*(endlong + startlong);
        latmin = fmin(endlat, startlat);
        latmax = fmax(endlat, startlat);
        longmin = avlong - 0.5*latdif;
        longmax = avlong + 0.5*latdif;
    }
    else    {
        longmin = fmin(endlong, startlong);
        longmax = fmax(endlong, startlong);
        latmin = fmin(endlat, startlat);
        latmax = fmax(endlat, startlat);
    }

    printf("New values\n");
    printf("Minimum longitude value: %lf\n", longmin);
    printf("Maximum longitude value: %lf\n", longmax);
    printf("Minimum latitude value: %lf\n", latmin);
    printf("Maximum latitude value: %lf\n", latmax);
    printf("\n");


    // Find the points within the given area

    for(i = 0; i < length; i++) {
        if(x[i] > longmax || x[i] < longmin || y[i] > latmax || y[i] < latmin) {
            newlength--;
            x[i] = 1000.0;
        }
    }

    xt = (double*) malloc(sizeof(double)*newlength);
    yt = (double*) malloc(sizeof(double)*newlength);

    s = 0;
    for(i = 0; i < length; i++) {
        if(x[i] < 1000.0){
            xt[s] = x[i];
            yt[s] = y[i];
            s++;
        }
    }

    free(x);
    free(y);

    length = newlength;

    x = (double*) malloc(sizeof(double)*length);
    y = (double*) malloc(sizeof(double)*length);

    for (i = 0; i < length; i++) {
        x[i] = xt[i];
        y[i] = yt[i];
    }
    free(xt);
    free(yt);

    // print out the data within the given area
    for (i = 0; i < length; i++) {
        fprintf(out6, "%lf %lf\n", x[i], y[i]);
    }
    fclose(out6);

    // ***************************************************************************************
    // Transform the data to a 2D metric grid system
    // Calculate the number of gridpoints N
    // **************************************************************************************
    degrad = 1.0/180.0;
    xmax = pi*R*degrad*(longmax - longmin);
    ymax = pi*R*degrad*(latmax - latmin);

    while (choise != 'y'){

        printf("\n");
        printf("Enter the prefered distance (in meters) between grid points:  ");
        scanf("%lf", &gridsize);
        printf("\n");

        N = ((int) (ymax/gridsize)) + 1;
        printf("The given distance gives a %i x %i grid\n", N, N);
        printf("Press 'y' to continue, or press 'n' to change the given distance:  ");
        scanf(" %c", &choise);
        printf("\n");
    }

    fprintf(out7, "Grid dimension: %i\n", N);
    fprintf(out7, "Cell size [m]: %lf\n", gridsize);
    fclose(out7);


    //For plotting the reduced dataset
    fprintf(gpl2, "cd '%s'\n",dir);
    fprintf(gpl2, "set term png enhanced\n");
    fprintf(gpl2, "unset key\n");
    fprintf(gpl2, "set xrange [0:%i]\n", N);
    fprintf(gpl2, "set yrange [0:%i]\n", N);
    fprintf(gpl2, "set xlabel \"i\"\n");
    fprintf(gpl2, "set ylabel \"j\" rotate by 0\n");
    fprintf(gpl2, "set out 'partial_data.png'\n");
    fprintf(gpl2, "plot 'coord_trans.dat' using 1:2 with dots\n");
    fprintf(gpl2, "exit");
    fclose(gpl2);


    // For plotting multiple realizations with dataset, Need the value of N before writing
    fprintf(gpl4, "cd '%s' \n", dir);
    fprintf(gpl4, "set term png enhanced\n");
    fprintf(gpl4, "set key outside\n");
    fprintf(gpl4, "set xrange [0:%i]\n", N+1);
    fprintf(gpl4, "set yrange [0:%i]\n", N+1);
    fprintf(gpl4, "set xlabel \"i\"\n");
    fprintf(gpl4, "set ylabel \"j\" rotate by 0\n");
    fprintf(gpl4, "set out 'diagonal_with_dataset.png'\n");
    fprintf(gpl4, "plot 'coord_trans.dat' using 1:2 notitle with dots lc rgb 'black', ");
    for(i = 0; i < times; i++) {
        fprintf(gpl4, "'minimumvalues%i.dat' using 1:2 title \"%.2f\"",i, delta*i);
        if(i < times-1)
            fprintf(gpl4, ", ");
    }
    fprintf(gpl4, "\n");
    fprintf(gpl4, "exit");
    fclose(gpl4);




    // Allocatation
    grid2d = (double**) malloc(sizeof(double*)*(N+1));
    for (i = 0; i < N+1; i++) {
        grid2d[i] = (double*) malloc(sizeof(double)*(N+1));
    }
    // Initializing grid
    for (i = 0; i < N+1; i++) {
        for (j = 0; j < N+1; j++) {
            grid2d[i][j] = 0.0;
        }
    }

    // Transform the data
    for(i = 0; i < length; i++) {
        x[i] = (N-1)*(( pi*R*degrad*(x[i] - longmin) )/xmax);
        y[i] = (N-1)*(( pi*R*degrad*(y[i] - latmin) )/ymax);
    }

    // Count and write results for each gridpoint
    for(i = 0; i < length; i++) {

        // Find the lower left corner
        xint = (int)(x[i]);
        yint = (int)(y[i]);

        // Find the distances to each corner of the surrounding square

        r1 = sqrtf( (x[i]- xint)*(x[i]- xint) + (y[i] - yint)*(y[i] - yint) );
        r2 = sqrtf( (x[i]- xint)*(x[i]- xint) + (y[i] - (yint+1))*(y[i] - (yint+1)) );
        r3 = sqrtf( (x[i]- (xint+1))*(x[i]- (xint+1)) + (y[i] - (yint+1))*(y[i] - (yint+1)) );
        r4 = sqrtf( (x[i]- (xint+1))*(x[i]- (xint+1)) + (y[i] - yint)*(y[i] - yint) );

        rtot = r1 + r2 + r3 + r4;

        w1 = (rtot - r1)/(3.0*rtot);
        w2 = (rtot - r2)/(3.0*rtot);
        w3 = (rtot - r3)/(3.0*rtot);
        w4 = (rtot - r4)/(3.0*rtot);

        grid2d[xint][yint] = grid2d[xint][yint] + w1;
        grid2d[xint][yint+1] = grid2d[xint][yint+1] + w2;
        grid2d[xint+1][yint+1] = grid2d[xint+1][yint+1] + w3;
        grid2d[xint+1][yint] = grid2d[xint+1][yint] + w4;

        fprintf(out3,"%lf %lf\n", x[i], y[i]);
    }


    for (i = 0; i < N+1; i++) {
        for (j = 0; j < N+1; j++) {

            // Print all grid values
            fprintf(out1,"%i %i %lf\n", i, j, grid2d[i][j]);
            // Print all non-zero grid values
            if(grid2d[i][j] > 0.0000001)
                fprintf(out2,"%i %i %lf\n", i, j, grid2d[i][j]);
        }
    }

    fclose(out1);
    fclose(out2);
    fclose(out3);

    // Plot the new system
    sprintf(gnu,"%s %s","gnuplot", gpl2n);
    system(gnu);

    for(i = 0; i < times; i++) {

        alpha = delta*i;

        pathscape(dir, N, alpha, 0, 0, N, N);
        repl(dir, N, i);
        pathlength = calc_pathlength(dir, N, i);

        if  (i == 0)
            firstpath = pathlength;

        printf("%lf \n", pathlength);

        fprintf(out4, "%lf %lf %lf\n", alpha, pathlength, pathlength/firstpath);
    }

    fclose(out4);

    sprintf(gnu, "%s %s", "gnuplot", gpl3n);
    system(gnu);

    sprintf(gnu, "%s %s", "gnuplot", gpl4n);
    system(gnu);

    sprintf(gnu, "%s %s", "gnuplot", gpl5n);
    system(gnu);

    sprintf(gnu, "%s %s", "gnuplot", gpl6n);
    system(gnu);

    return 0;
}

