/*
**  This version of pathscapeorg.c is ment to be used for detecing patterns, it is called by grid.c
**    The function patchsape() takes a number of inputs
**        - Result directory
**        - Grid size N ( NxN)
**        - The parameter alpha
*/

#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include <math.h>

void fillGrid();
void update();
double ranIBM();

int IBM = 16371;                                //Random seed
double rinv = 0.5/(((long int)1 << 31)-1.0);

FILE *grid;

int N;
int i, j, k;
double read;
int toss;

double infty = 100.0;
double inftybound = 10000.0;
double eps = 0.00001;
double pot;

bool condition;

char dir[150];

double **dens;
double ***v;
double ***tv;

void pathscape (char directory[], int n, double alpha, int x0, int y0, int x1, int y1) {

    sprintf(dir, directory);
    pot = alpha;
    N = n;

    double dnew, dold = 0.0;

    //Names
    char gridn[200], out1n[200], out2n[200], gpl1n[200];
    sprintf(gridn, "%s%s", dir, "\\gridvalues.dat");
    sprintf(out1n, "%s%s", dir, "\\vij.dat");
    sprintf(gpl1n, "%s%s", dir, "\\plot.txt");

    // FILES

    //input
    grid = fopen(gridn,"r");

    //output
    FILE *out1 = fopen(out1n, "w");

    //plotscript
    FILE *gpl1 = fopen(gpl1n,"w");


    // Allocate arrays
    dens = (double**) malloc(sizeof(double*)*(N+1));
    for(i = 0; i < N+1; i++) {
        dens[i] = (double*) malloc(sizeof(double)*(N+1));
    }

    v = (double***) malloc(sizeof(double**)*2);
    tv = (double***) malloc(sizeof(double**)*2);
    for (i = 0; i < 2; i++) {
        v[i] = (double**) malloc(sizeof(double*)*(N+1));
        tv[i] = (double**) malloc(sizeof(double*)*(N+1));
    }
    for (i = 0; i < 2; i++) {
        for (j = 0; j < N+1; j++) {
            v[i][j] = (double*) malloc(sizeof(double)*(N+1));
            tv[i][j] = (double*) malloc(sizeof(double)*(N+1));
        }
    }

    fillGrid();
    fclose(grid);

    v[0][x0][y0] = 0.0;
    tv[0][x0][y0] = 0.0;
    v[1][x1][y1]= 0.0;
    tv[1][x1][y1] = 0.0;


    condition = false;

    int itcount = 0;
    while(condition != true) {
        dnew = 0.0;
        itcount++;
        update();

        for (i = 1; i < N; i++) {
            for (j = 1; j < N; j++) {
                dnew = dnew + v[0][i][j] + v[1][i][j];
            }
        }

        if(fabs(dnew - dold) <= eps)
            condition = true;
        dold = dnew;

        if(itcount % 100 == 0)
            printf("Iterations: %i \n", itcount);
    }


    for (i = 0; i < N+1; i++) {
        for (j = 0; j < N+1; j++) {
            fprintf(out1,"%i %i %lf\n", i, j, v[0][i][j] + v[1][i][j]);
        }
    }
    fclose(out1);


    /*
    **  Create plotscript and plot
    */

    fprintf(gpl1,"cd '%s'\n",dir);
    fprintf(gpl1,"set term pdf\n");
    fprintf(gpl1,"set out 'pathscape.pdf'\n");
    fprintf(gpl1,"set view map\n");
    fprintf(gpl1,"set palette gray\n");
    fprintf(gpl1,"splot 'vij.dat' using 1:2:3 with image\n");
    fprintf(gpl1,"exit\n");
    fclose(gpl1);

    char plot[200];
    sprintf(plot,"%s %s","gnuplot",gpl1n);
    system(plot);
}

void fillGrid(){


    // Read in the dataset and inverse the values
    for (i = 0; i < N+1; i++) {
        for(j = 0; j < N+1; j++) {
            fscanf(grid, "%i %i %lf\n", &toss, &toss, &read);
            if(read > 0.000001)
                dens[i][j] = pow(1.0/read, pot);
            else
                dens[i][j] = pow(infty,pot);
        }
    }

    for (i = 0; i < 2; i++) {
        for (j = 0; j < N+1; j++) {
            for (k = 0; k < N+1; k++) {

                v[i][j][k] = 0.0;

                if  (j == 0 || j == N || k == 0  || k == N )
                    v[i][j][k] = inftybound;

                tv[i][j][k] = v[i][j][k];
            }
        }
    }

}

void update() {

    double minv, up1, up2, up3, up4, up5, up6, up7, up8;

    for (i = 0; i < 2; i++) {
        for (j = 1; j < N; j++) {
            for (k = 1; k < N; k++) {

                tv[i][j][k] = v[i][j][k];
            }
        }
    }

    for (i = 0; i < 2; i++) {
        for (j = 1; j < N; j++) {
            for (k = 1; k < N; k++) {

                up1 = (dens[j][k+1] + dens[j][k])/2.0 + tv[i][j][k+1];
                up2 = (dens[j][k-1] + dens[j][k])/2.0 + tv[i][j][k-1];
                up3 = (dens[j+1][k] + dens[j][k])/2.0 + tv[i][j+1][k];
                up4 = (dens[j-1][k] + dens[j][k])/2.0 + tv[i][j-1][k];
                up5 = sqrtf(2)*(dens[j+1][k+1] + dens[j][k])/2.0 + tv[i][j+1][k+1];
                up6 = sqrtf(2)*(dens[j+1][k-1] + dens[j][k])/2.0 + tv[i][j+1][k-1];
                up7 = sqrtf(2)*(dens[j-1][k+1] + dens[j][k])/2.0 + tv[i][j-1][k+1];
                up8 = sqrtf(2)*(dens[j-1][k-1] + dens[j][k])/2.0 + tv[i][j-1][k-1];


                minv = up1;
                if(up2 <= minv)
                    minv = up2;
                if(up3 <= minv)
                    minv = up3;
                if(up4 <= minv)
                    minv = up4;
                if(up5 <= minv)
                    minv = up5;
                if(up6 <= minv)
                    minv = up6;
                if(up7 <= minv)
                    minv = up7;
                if(up8 <= minv)
                    minv = up8;

                v[i][j][k] = minv;
            }
        }
    }
}


double ranIBM() {
    IBM *= 16807;
    return (rinv*IBM + 0.5);
}
