
/*
**  grid.c
**      - Works as the main file for the project
**      - Folder for results must be specifed in line 28
**      - The dataset contating coordiantes must be
**        translated to a suitable form
**      - The prefered size of area to be analyzed
**         must be specified in lines 71 - 74
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "pathscapeorg.c"
#include "replot.c"
#include "transback.c"
#include "calclength.c"

void bubblesorting(double arr[], int nElements);

double *x, *y, *xt, *yt, *sorted;
double **grid2d, **pscape;

int main () {

    // Directory of results
    char dir[150] = "specify\the\result\folder\here";

    /*
    ** Parameters
    **  N          :  Dimension of grid N*N
    **  length     :  number of datapoints
    **  R          :  Earths radius [m]
    **  pi         :  pi
    **  infty      :  Value of "infinity"
    **  alpha      :  Parameter for adjusting the energylandscape
    **  scalepow   :  Scaling parameter for energies
    **  scaleconst :  Scale constant, to enhance differences in energy
    **  alenght    :  Interval length on boundary,
    **  times      :  The number of paths included in the pattern
    **
    */

    int N; // User specified
    int length = 0;
    int R = 6371000;
    double pi = 3.14159265359;
    double infty = 10000.0;
    double alpha = 0.8;
    double scalepow = 0.8;
    double scaleconst = 1.0/10.0;
    int alength = 10;
    int times = 1;

    // Other Variables
    int i, j, k, l, m, xint, yint, change;
    int newlength = length, s, drop, nside;
    int ni, nj, nP, startx, starty, xs, ys;
    double xi, yi, xmax, ymax, xmin, ymin;
    double degrad, avspeed;
    double longmin = 10000, longmax = 0;
    double latmin = 10000, latmax = 0;
    double startlong, startlat, endlong, endlat;
    double longdif, latdif, avlong, avlat;
    double dimension, gridsize, pathlength;
    double w1, w2, w3, w4, r1, r2, r3, r4, rtot;
    double dropd;
    char name[200], gnu[200];
    char choise = 'n';

    // SET THE BOUNDARY VALUES
    // Specify the area of interest by its
    // bounday values given in geodetic coordinates
    startlong = 0.0;
    endlong = 0.0;
    startlat = 0.0;
    endlat = 0.0;

    /*
    **  Creating Data Files for results and information
    */
    char datan[200], out1n[200], out2n[200], out3n[200];
    char minin[200], nPpath[200];
    char gpl1n[200], gpl2n[200], gpl3n[200];

    // INPUT FILE
    sprintf(datan, "%s%s",dir,"\\coords.dat");

    // OUTPUT ADN PLOTSCRIPTS
    sprintf(out1n, "%s%s",dir,"\\gridvalues.dat");
    sprintf(out2n, "%s%s",dir,"\\coords_reduced.dat");
    sprintf(out3n, "%s%s",dir,"\\coord_trans.dat");
    sprintf(minin, "%s%s",dir,"\\minimumvalues100.dat");
    sprintf(nPpath,"%s%s",dir,"\\number_val.dat");
    sprintf(gpl1n, "%s%s",dir,"\\plot_full.txt");
    sprintf(gpl2n, "%s%s",dir,"\\plot_part.txt");
    sprintf(gpl3n, "%s%s",dir,"\\plot_optimal_paths.txt");

    FILE *data = fopen(datan, "r"); // input coordinates
    FILE *out1 = fopen(out1n, "w"); // Gridvalues
    FILE *out2 = fopen(out2n, "w"); // reduced coordinate-set
    FILE *out3 = fopen(out3n, "w"); // Transformed coordinates
    FILE *gpl1 = fopen(gpl1n,"w");  // Gnuplot script
    FILE *gpl2 = fopen(gpl2n,"w");  // Gnuplot script
    FILE *gpl3 = fopen(gpl3n,"w");  // Gnuplot script

    /*
    ** Allocation
    */
    x = (double*) malloc(sizeof(double)*length);
    y = (double*) malloc(sizeof(double)*length);

    /*
    ** Reading - The reading of coordinates, depends of form of file
    */
    for (i = 0; i < length; i++) {

        fscanf(data, "%lf %lf\n", &xi, &yi);
        x[i] = xi;
        y[i] = yi;

        if (xi >= longmax)
            longmax = xi;
        if (xi <= longmin)
            longmin = xi;
        if (yi >= latmax)
            latmax = yi;
        if (yi <= latmin)
            latmin = yi;
    }
    fclose(data);

    /*
    **  Create plotscripts
    */

    // For the original dataset
    fprintf(gpl1, "cd '%s'\n",dir);
    fprintf(gpl1, "set term png enhanced\n");
    fprintf(gpl1, "unset key\n");
    fprintf(gpl1, "set out 'original_data.png'\n");
    fprintf(gpl1, "plot 'coords_routes_sorted.dat' using 1:2\n");
    fprintf(gpl1, "exit");
    fclose(gpl1);

    //For the reduced dataset
    fprintf(gpl2, "cd '%s'\n",dir);
    fprintf(gpl2, "set term png enhanced\n");
    fprintf(gpl2, "unset key\n");
    fprintf(gpl2, "set out 'partial_data.png'\n");
    fprintf(gpl2, "plot 'coord_trans.dat' using 1:2\n");
    fprintf(gpl2, "exit");
    fclose(gpl2);


    //Plot the original data:
    sprintf(gnu,"%s %s","gnuplot", gpl1n);
    system(gnu);

    longdif = fabs(endlong - startlong);
    latdif = fabs(endlat - startlat);

    if  (longdif > latdif)  {
        avlat = 0.5*(endlat + startlat);
        longmin = fmin(endlong, startlong);
        longmax = fmax(endlong, startlong);
        latmin =  avlat - 0.5*longdif;
        latmax =  avlat + 0.5*longdif;
    }
    else if (longdif < latdif)   {
        avlong = 0.5*(endlong + startlong);
        latmin = fmin(endlat, startlat);
        latmax = fmax(endlat, startlat);
        longmin = avlong - 0.5*latdif;
        longmax = avlong + 0.5*latdif;
    }
    else    {
        longmin = fmin(endlong, startlong);
        longmax = fmax(endlong, startlong);
        latmin = fmin(endlat, startlat);
        latmax = fmax(endlat, startlat);
    }

    // Count the number of datapoints in the desired area
    // and make new arrays of the new length

    for(i = 0; i < length; i++) {
        if(x[i]>longmax || x[i]<longmin || y[i]>latmax || y[i]<latmin) {
            newlength--;
            x[i] = 1000.0;
        }
    }

    xt = (double*) malloc(sizeof(double)*newlength);
    yt = (double*) malloc(sizeof(double)*newlength);

    // Set the length of x and y to the new length and put back points

    s = 0;
    for(i = 0; i < length; i++) {
        if(x[i] < 1000.0){
            xt[s] = x[i];
            yt[s] = y[i];
            s++;
        }
    }
    free(x);
    free(y);

    length = newlength;

    x = (double*) malloc(sizeof(double)*length);
    y = (double*) malloc(sizeof(double)*length);

    for (i = 0; i < length; i++) {
        x[i] = xt[i];
        y[i] = yt[i];
    }
    free(xt);
    free(yt);

    /*
    **  Print out the (long,lat) - coordinates in the limited area
    */
    for (i = 0; i < length; i++) {
        fprintf(out2, "%lf %lf\n", x[i], y[i]);
    }
    fclose(out2);

    /*
    **  - Transform the maximum points from [deg] to [m]
    **    to get the span of the area
    **  - Ask for grid size in meters
    **  - Calculate the number of gridpoints
    */

    degrad = 1.0/180.0;
    xmax = pi*R*degrad*(longmax - longmin);
    ymax = pi*R*degrad*(latmax - latmin);

    while (choise != 'y')   {

    printf("\n");
    printf("Enter the prefered distance (in [m]) between grid points:");
    scanf("%lf", &gridsize);
    printf("\n");

    N = ((int) (ymax/gridsize));
    printf("The given distance gives a %i x %i grid\n", N, N);
    printf("Press 'y' to continue, or press 'n' to change the distance:");
    scanf(" %c", &choise);
    printf("\n");
    }

    // Create plotscripts that need the value of N
    // For plotting the optimal paths
    fprintf(gpl3, "cd '%s'\n", dir );
    fprintf(gpl3, "set term png enhanced\n");
    fprintf(gpl3, "unset key\n");
    fprintf(gpl3, "set xrange [0:%i]\n", N);
    fprintf(gpl3, "set yrange [0:%i]\n", N);
    fprintf(gpl3, "set xlabel \"i\"\n");
    fprintf(gpl3, "set ylabel \"j\" rotate by 0\n");
    fprintf(gpl3, "set out 'multiple_paths.png'\n");
    fprintf(gpl3, "plot 'coord_trans.dat' using 1:2 with dots, ");
    for(i = 0; i < times; i++) {
      fprintf(gpl3,"'minimumvalues%i.dat' using 1:2 lc rgb 'black'", i);
      if(i < times - 1)
         fprintf(gpl3, ", ");
    }
    fprintf(gpl3, "\n");
    fprintf(gpl3, "exit");
    fclose(gpl3);

    /*
    **  Allocate memory for arrays, which size is given by N
    */

    grid2d = (double**) malloc(sizeof(double*)*(N+1));
    pscape = (double**) malloc(sizeof(double*)*(N+1));
    for (i = 0; i < N+1; i++) {

        grid2d[i] = (double*) malloc(sizeof(double)*(N+1));
        pscape[i] = (double*) malloc(sizeof(double)*(N+1));
    }

    // Initializing grid
    for (i = 0; i < N+1; i++) {
        for (j = 0; j < N+1; j++) {

            grid2d[i][j] = 0.0;
            pscape[i][j] = 0.0;
        }
    }

    /*
    ** Transformation
    */

    for (i = 0; i < length; i++) {

        x[i] = (N-1)*(( pi*R*degrad*(x[i] - longmin) )/xmax);
        y[i] = (N-1)*(( pi*R*degrad*(y[i] - latmin) )/ymax);
    }

    // Count the number of objects within each grid cell
    // and write the result to file

    for(i = 0; i < length; i++) {

        // Find the lower left corner
        xint = (int)(x[i]);
        yint = (int)(y[i]);

        // Find the distances to each corner of the surrounding square

        r1 = sqrtf((x[i]-xint)*(x[i]-xint) +
                    (y[i]-yint)*(y[i]-yint));

        r2 = sqrtf((x[i]-xint)*(x[i]-xint) +
                    (y[i]-(yint+1))*(y[i]-(yint+1)));

        r3 = sqrtf((x[i]-(xint+1))*(x[i]-(xint+1)) +
                   (y[i]-(yint+1))*(y[i]-(yint+1)));

        r4 = sqrtf((x[i]-(xint+1))*(x[i]-(xint+1)) +
                   (y[i]-yint)*(y[i]-yint));

        rtot = r1 + r2 + r3 + r4;

        w1 = (rtot - r1)/(3*rtot);
        w2 = (rtot - r2)/(3*rtot);
        w3 = (rtot - r3)/(3*rtot);
        w4 = (rtot - r4)/(3*rtot);

        grid2d[xint][yint] += w1;
        grid2d[xint][yint+1] += w2;
        grid2d[xint+1][yint+1] += w3;
        grid2d[xint+1][yint] += w4;

         fprintf(out3,"%lf %lf \n", x[i], y[i]);
    }

    for (i = 0; i < N+1; i++) {
        for (j = 0; j < N+1; j++) {
            fprintf(out1,"%i %i %.15f\n", i, j, grid2d[i][j]);
        }
    }

    // Plot the coordinates within the area of interest
    sprintf(gnu, "%s %s", "gnuplot", gpl2n);
    system(gnu);

    free(x);
    free(y);

    fclose(out1);
    fclose(out3);

    // Parameters

    int tempalength = alength, blength = alength, an, nnside;
    an = N/alength + 1;
    int lastlength =  N - (an-1)*alength;
    int ans6 = 6*an*an;
    int sx0[alength], sy0[alength], sx1[alength], sy1[alength];

    char totaln[200], totaloutn[200], totalalln[200];
    char newoutn[200], numvals[200], tralan[200];

    sprintf(totaln, "%s%s", dir, "\\vij.dat");
    sprintf(numvals, "%s%s", dir, "\\number_val.dat");
    sprintf(newoutn, "%s%s", dir, "\\sorted_minpaths.dat");
    sprintf(totalalln, "%s%s", dir, "\\total_all.dat");

    FILE *ut2 = fopen(totalalln, "w");

    // loop over all pairs of axes
    for(nside = 0; nside < 6; nside++) {
        sprintf(totaloutn, "%s%s%i%s", dir, "\\total", nside, ".dat");
        FILE *ut = fopen(totaloutn, "w");

        // loop over all intervals startpoints   < an
        for(j = 0; j < an; j++) {
            // loop over all intervals endpoints  < an
            for(s = 0; s < an; s++){

                if(j == an-1)
                    alength = lastlength;
                else
                    alength = tempalength;
                if(s == an-1)
                    blength = lastlength;
                else
                    blength = tempalength;

                for(i = 0; i < alength; i++) {
                        if(nside == 0) {
                            sx0[i] = i + j*tempalength;
                            sy0[i] = 0;
                        }
                        if(nside == 1) {
                           sx0[i] = 0;
                           sy0[i] = i + j*tempalength;
                        }
                        if(nside == 2) {
                            sx0[i] = i + j*tempalength;
                            sy0[i] = 0;
                        }
                        if(nside == 3) {
                           sx0[i] = 0;
                           sy0[i] = i + j*tempalength;
                        }
                        if(nside == 4) {
                            sx0[i] = i + j*tempalength;
                            sy0[i] = N;
                        }
                        if(nside == 5) {
                           sx0[i] = N;
                           sy0[i] = i + j*tempalength;
                        }
                }
                for(i = 0; i < blength; i++) {
                        if(nside == 0) {
                            sx1[i] = i + s*tempalength;
                            sy1[i] = N;
                        }
                        if(nside == 1)  {
                            sx1[i] = N;
                            sy1[i] = i + s*tempalength;
                        }
                        if(nside == 2) {
                            sx1[i] = 0;
                            sy1[i] = i + s*tempalength;
                        }
                        if(nside == 3)  {
                            sx1[i] = i + s*tempalength;
                            sy1[i] = N;
                        }
                        if(nside == 4) {
                            sx1[i] = N;
                            sy1[i] = i + s*tempalength;
                        }
                        if(nside == 5)  {
                            sx1[i] = i + s*tempalength;
                            sy1[i] = 0;
                        }
                }

                printf("nside: %i j: %i s: %i\n", nside, j, s);

        // Generate a pathscape
        pathscape(dir, sx0, sy0, sx1, sy1, alength, blength, N, alpha);

        // Run repl() to get the minimumpath in the pathscape
        repl(dir, N, 100);

        // Open the file "minimumvalues.dat"
        // to get the energy of the minimumpath
        FILE *minpathFile = fopen(minin, "r");
        fscanf(minpathFile, "%i %i %lf\n", &drop, &drop, &xmin);
        fclose(minpathFile);

        // Run calc_pathlength() to get the minimumspath length
        // For now, the function only retuns the number of points
        // on the minimumpath - 1
        pathlength = calc_pathlength(dir, N, 100);
        printf("pathlength returned : %lf \n", pathlength);
        if(pathlength == 0.0001)
            xmin = 100;

        fprintf(ut, "%i %i %i %lf %lf\n", nside, j, s, xmin, pathlength);
        fprintf(ut2, "%i %i %i %lf %lf\n", nside, j, s, xmin, pathlength);

        }
      }
        fclose(ut);

    }

    fclose(ut2);


    FILE *ina = fopen(totalalln, "r");


    double unsorted[ans6], tunsorted[ans6];
    int first[ans6], second[ans6], tside[ans6];


    sorted = (double*) malloc(sizeof(double)*(ans6));

    for(i = 0; i < ans6; i++) {
        fscanf(ina,"%i %i %i %lf %lf\n",&nnside,&ni,&nj,&xi,&pathlength);
        unsorted[i] = xi/(scaleconst*pow(pathlength, scalepow));
        tunsorted[i] = xi/(scaleconst*pow(pathlength, scalepow));
        first[i] = ni;
        second[i] = nj;
        tside[i] = nnside;
        }
        fclose(ina);

        bubblesorting(unsorted, ans6);

        printf("sorted: \n");
        for(i = 0; i < ans6; i++) {
            printf("%lf\n", sorted[i]);
        }

        FILE *newout = fopen(newoutn,"w");

        for(i = 0; i < ans6; i++) {

            for(j = 0; j < ans6; j++) {
                if(tunsorted[i] == sorted[j])
                    s = j;
            }
        fprintf(newout, "%i %i %i %.15f %i\n",tside[i],first[i],
                second[i],tunsorted[i],s);
        }
        fclose(newout);

        int wpath, npaths;

        /*
        **  The actual pathscapes are not stored,
        **  only their optimal path information.
        **  The pathscapes chosen for the pattern
        **  must therefore be run again.
        */

        for(npaths = 0; npaths < times; npaths++) {
            FILE *inagain = fopen(newoutn, "r");
            for(i = 0; i < ans6; i++) {
    fscanf(inagain,"%i %i %i %lf %i\n",&nnside,&ni,&nj,&xi,&wpath);
                if(wpath == npaths){
                    j = ni;
                    s = nj;
                    nside = nnside;
                }
            }
                if(j == an - 1)
                    alength = lastlength;
                else
                    alength = tempalength;
                if(s == an - 1)
                    blength = lastlength;
                else
                    blength = tempalength;

                for(i = 0; i < alength; i++) {
                    if(nside == 0) {
                        sx0[i] = i + j*tempalength;
                        sy0[i] = 0;
                    }
                    if(nside == 1)  {
                        sx0[i] = 0;
                        sy0[i] = i + j*tempalength;
                    }
                    if(nside == 2) {
                        sx0[i] = i + j*tempalength;
                        sy0[i] = 0;
                    }
                    if(nside == 3) {
                        sx0[i] = 0;
                        sy0[i] = i + j*tempalength;
                    }
                    if(nside == 4) {
                        sx0[i] = i + j*tempalength;
                        sy0[i] = N;
                    }
                    if(nside == 5) {
                        sx0[i] = N;
                        sy0[i] = i + j*tempalength;
                    }
                }
                for(i = 0; i < blength; i++) {
                    if(nside == 0) {
                        sx1[i] = i + s*tempalength;
                        sy1[i] = N;
                    }
                    if(nside == 1)  {
                        sx1[i] = N;
                        sy1[i] = i +s*tempalength;
                    }
                    if(nside == 2) {
                        sx1[i] = 0;
                        sy1[i] = i + s*tempalength;
                    }
                    if(nside == 3)  {
                        sx1[i] = i + s*tempalength;
                        sy1[i] = N;
                    }
                    if(nside == 4) {
                        sx1[i] = N;
                        sy1[i] = i + s*tempalength;
                    }
                    if(nside == 5)  {
                        sx1[i] = i + s*tempalength;
                        sy1[i] = 0;
                    }
                }

        pathscape(dir, sx0, sy0, sx1, sy1, alength, blength, N, alpha);
        repl(dir, N, npaths);
        fclose(inagain);
      }

        sprintf(gnu,"%s %s", "gnuplot", gpl3n);
        system(gnu);


    return 0;
}

/*
**  Simple implementation of the bubble sorting algorithm,
**  used to sort optimal paths by energy
*/
void bubblesorting(double arr[], int nElements) {

    double temp;
    int i, j;

    for (i = 0; i < nElements - 1; i++) {
        for (j = 0; j < nElements - 1 - i; j++) {

            if(arr[j] >= arr[j+1]) {
                temp = arr[j+1];
                arr[j+1] = arr[j];
                arr[j] = temp;
            }
        }
    }

    for (i = 0; i < nElements; i++) {
        sorted[i] = arr[i];
    }
}
