/*
**  replot.c
**      - called from grid.c
**      - used to localize minimupath in pathscapae
**      - originally used to replot data,
**      - plots the minimumpath
*/


#include <stdio.h>
#include <stdlib.h>

double **dens;

void repl(char directory[], int N, int npaths) {

    //Parameters

    double infty = 10000.0;     // "infinity"
    double perc = 0.000000001;  // "zero"

    //Directory

    char dir[150];
    sprintf(dir, directory);

    // Help parameters
    int i, j, toss;
    double read, minval  = 5*infty;

    /*
    ** Creating and opening datafiles and gnuplot script
    */
    char inn[200], inNn[200];
    char out1n[200], out2n[200], out3n[200];
    char gpln[200], plot[200];
    sprintf(inn, "%s%s", dir, "\\vij.dat");
    sprintf(out1n, "%s%s", dir, "\\part_vij.dat");
    sprintf(out2n, "%s%s%i%s", dir, "\\minimumvalues",npaths,".dat");
    sprintf(out3n, "%s%s", dir, "\\number_val.dat");
    sprintf(gpln, "%s%s", dir, "\\plotpaths.txt");
    sprintf(plot, "%s %s", "gnuplot", gpln);

    FILE *in = fopen(inn,"r");
    FILE *out1 = fopen(out1n,"w");
    FILE *out2 = fopen(out2n,"w");
    FILE *out3 = fopen(out3n,"w");
    FILE *gpl = fopen(gpln,"w");



    // Plot script
    fprintf(gpl, "cd '%s'\n", dir);
    fprintf(gpl, "set term pdf\n");
    fprintf(gpl, "set out 'optimalpath.pdf'\n");
    fprintf(gpl, "set view map\n");
    fprintf(gpl, "set palette gray\n");
    fprintf(gpl, "set xrange [0:%i]\n", N+1);
    fprintf(gpl, "set yrange [0:%i]\n", N+1);
    fprintf(gpl, "splot 'part_vij.dat' using 1:2:3 with image\n");
    fprintf(gpl, "exit\n");
    fclose(gpl);



    dens = (double**) malloc(sizeof(double*)*N+1);
    for (i = 0; i < N+1; i++) {
        dens[i] = (double*) malloc(sizeof(double)*N+1);
    }

    // Read in data and find minimum
    for (i = 0; i < N+1; i++) {
        for (j = 0; j < N+1; j++) {

            fscanf(in, "%i %i %lf\n", &toss, &toss, &read);
            dens[i][j] = read;

            if  (dens[i][j] < minval)
                minval = dens[i][j];
        }
    }

    double nextminval = 5*infty;
    for (i = 0; i < N+1; i++) {
        for (j = 0; j < N+1; j++) {

            if(dens[i][j] < nextminval && dens[i][j] > minval)
                nextminval = dens[i][j];
        }
    }


    // Print new data
    for (i = 0; i < N+1; i++) {
        for (j = 0; j < N+1; j++) {

            if  (dens[i][j] > (1.0 + perc)*minval)
                dens[i][j] = infty;

            fprintf(out1, "%i %i %lf\n", i, j, dens[i][j]);
        }
    }

    int counter = 0;

    for(i = 0; i < N+1; i++){
        for(j = 0; j < N+1; j++) {
            if(dens[i][j] <= minval*(1.0 + perc)){
            fprintf(out2,"%i %i %.15f\n", i, j, dens[i][j]);
            counter++;
            }
        }
    }

    fprintf(out3,"%i\n", counter);
    printf("%i", counter);

    free(dens);

    fclose(in);
    fclose(out1);
    fclose(out2);
    fclose(out3);

    system(plot);

}
