#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "pathscapeorg.c"
#include "replot.c"
#include "transback.c"

double *x, *y, *xt, *yt;
double **grid2d;

int main() {

    /*
    ** Parameters
    **  N       :   Dimension of grid N*N
    **  length  :   number of datapoints ( - 1)
    **  R       :   Earths radius [m]
    **  pi      :   pi
    */

    int N;                  // The dimension of the grid is determined later in the program
    int length = 86765;
    int R = 6371000;
    double pi = 3.14159265359;
    double alpha = 0.05;

    /*
    **  Directory and filenames
    */
    char dir[150] = "RESULT\DIRECTORY\GOES\HERE";
    char datan[200], out1n[200], out2n[200], out3n[200], out4n[200], out5n[200];

    char out6n[200], out7n[200], out8n[200], out9n[200], out10n[200], out11n[200];
    char gpl1n[200], gpl2n[200], gpl3n[200];

    sprintf(datan, "%s%s",dir,"\\coords_routes_sorted.dat");
    sprintf(out1n, "%s%s",dir,"\\gridvalues.dat");
    sprintf(out2n, "%s%s",dir,"\\gridvalues_reduced.dat");
    sprintf(out3n, "%s%s",dir,"\\coord_trans.dat");
    sprintf(out4n, "%s%s",dir,"\\physical_data.dat");
    sprintf(out5n, "%s%s",dir,"\\phys_reduced.dat");
    sprintf(out6n, "%s%s",dir,"\\coords_reduced.dat");
    sprintf(out7n, "%s%s",dir,"\\endpoints.dat");
    sprintf(out9n, "%s%s",dir,"\\endpoints_original.dat");
    sprintf(out10n, "%s%s",dir,"\\route_info.dat");
    sprintf(out11n, "%s%s",dir,"\\route_coordinates.dat");
    sprintf(gpl1n, "%s%s",dir,"\\plot_full.txt");
    sprintf(gpl2n, "%s%s",dir,"\\plot_part.txt");
    sprintf(gpl3n, "%s%s",dir,"\\plot_path.txt");


    //Help-parameters
    int i, j, xint, yint, change, newlength = length, s, chosenroute, route, croute = 0, year, month, day, hour, mins, sec, interval, drop;
    double xi, yi, longmin = 10000, longmax = 0, latmin = 10000, latmax = 0, xmax, ymax, xmin, ymin, degrad, avspeed;
    double startlong, startlat, endlong, endlat, longdif, latdif, avlong, avlat, dimension, gridsize;
    double w1, w2, w3, w4, r1, r2, r3, r4, rtot;
    double dropd;
    char name[1000];
    char gnu[1000];
    char choise = 'n';

    /*
    ** Allocation
    */
    x = (double*) malloc(sizeof(double)*length);
    y = (double*) malloc(sizeof(double)*length);


    /*
    ** Opening files
    */
    FILE *data = fopen(datan,"r");
    FILE *out1 = fopen(out1n,"w");
    FILE *out2 = fopen(out2n,"w");
    FILE *out3 = fopen(out3n,"w");
    FILE *out4 = fopen(out4n,"w");
    FILE *out5 = fopen(out5n,"w");
    FILE *out6 = fopen(out6n,"w");
    FILE *out7 = fopen(out7n,"w");
    FILE *out9 = fopen(out9n,"w");
    FILE *out10 = fopen(out10n,"w");
    FILE *out11 = fopen(out11n,"w");
    FILE *gpl1 = fopen(gpl1n,"w");
    FILE *gpl2 = fopen(gpl2n,"w");
    FILE *gpl3 = fopen(gpl3n,"w");

    /*
    ** Reading
    */
    for (i = 0; i < length; i++) {

        fscanf(data,"%lf %lf %lf %i %i %i %i %i %i %i %i %[^\n]s", &xi, &yi, &dropd, &drop, &drop, &drop, &drop, &drop, &drop, &drop, &drop, &name);
        x[i] = xi;
        y[i] = yi;

        if (xi >= longmax)
            longmax = xi;
        if (xi <= longmin)
            longmin = xi;
        if (yi >= latmax)
            latmax = yi;
        if (yi <= latmin)
            latmin = yi;
    }
    fclose(data);

    /*
    **  Create plotscripts
    */

    // For the original dataset
    fprintf(gpl1, "cd '%s'\n",dir);
    fprintf(gpl1, "set term png size 1024,1024\n");
    fprintf(gpl1, "unset key\n");
    fprintf(gpl1, "set out 'original_data.png'\n");
    fprintf(gpl1, "plot 'coords_routes_sorted.dat' using 1:2\n");
    fprintf(gpl1, "exit");
    fclose(gpl1);



    //For plotting of the path
    fprintf(gpl3, "cd '%s' \n", dir);
    fprintf(gpl3, "set term png size 1024,1024\n");
    fprintf(gpl3, "unset key\n");
    fprintf(gpl3, "set out 'path.png'\n");
    fprintf(gpl3, "plot 'route_coordinates.dat' using 1:2 with points pt 7 ps 1, 'route_coordinates.dat' using 1:2 with lines\n");
    fprintf(gpl3, "exit");
    fclose(gpl3);

    //Plot the original data:
    sprintf(gnu,"%s %s","gnuplot", gpl1n);
    system(gnu);

    printf("Dataset has been read and stored\n");
    printf("\n");
    printf("Minimum longitude value: %lf\n", longmin);
    printf("Maximum longitude value: %lf\n", longmax);
    printf("Minimum latitude value: %lf\n", latmin);
    printf("Maximum latitude value: %lf\n", latmax);
    printf("\n");

    printf("Which route should be estimated: \n");
    scanf("%i", &chosenroute);
    printf("\n");

    FILE *data2 = fopen(datan,"r");

    for (i = 0; i < length; i++) {
        fscanf(data2, "%lf %lf %lf %i %i %i %i %i %i %i %i %[^\n]s", &xi, &yi, &dropd, &year, &month, &day, &hour, &mins, &sec, &interval, &route, &name);

        if(chosenroute == route){
            croute++;
        }
    }

    printf("\n Route number %i contain %i datapoints\n", chosenroute, croute);

    fclose(data2);

    // Lager tabeller for verdiene av strrelse croute:
    double xar[croute], yar[croute], spear[croute];
    int ye[croute], mo[croute], da[croute], ho[croute], mi[croute], se[croute], ina[croute], ro[croute];
    char nameout[1000];

    j = 0; // Counter for the number of datapoints in route

    FILE *data3 = fopen(datan,"r");

    // Scanner inn p nytt og lagrer data for aktuell rute:
    for(i = 0; i < length; i++) {
        fscanf(data3,"%lf %lf %lf %i %i %i %i %i %i %i %i %[^\n]s", &xi, &yi, &dropd, &year, &month, &day, &hour, &mins, &sec, &interval, &route, &name);

        if(route == chosenroute){

                xar[j] = xi;
                yar[j] = yi;
                //spear[j] = intspeed; // dropped for now
                ye[j] = year;
                mo[j] = month;
                da[j] = day;
                ho[j] = hour;
                mi[j] = mins;
                se[j] = sec;
                ina[j] = interval;
                ro[j] = route;
                sprintf(nameout,name);

                j++;
        }
    }

    fclose(data3);

    fprintf(out10, "Ship Name: %s\n", nameout);
    fprintf(out10, "\n");
    fprintf(out10, "  lat      long     Date & time   \n");

    for(i = 0 ; i < croute; i++) {
        fprintf(out10, "%lf %lf %i %i %i %i %i %i\n", xar[i], yar[i], ye[i], mo[i], da[i], ho[i], mi[i], se[i]);
        fprintf(out11, "%lf %lf %i\n", xar[i], yar[i], ina[i]);
    }
    fclose(out10);
    fclose(out11);

    //Plot path
    sprintf(gnu,"%s %s","gnuplot", gpl3n);
    system(gnu);

    // Set the endpoints of the path:

    startlong = xar[0];
    startlat = yar[0];
    endlong = xar[croute - 1];
    endlat = yar[croute -1];

    longdif = fabs(endlong - startlong);
    latdif = fabs(endlat - startlat);

    if  (longdif > latdif)  {
        avlat = 0.5*(endlat + startlat);
        longmin = fmin(endlong, startlong);
        longmax = fmax(endlong, startlong);
        latmin =  avlat - 0.5*longdif;
        latmax =  avlat + 0.5*longdif;
    }
    else if (longdif < latdif)   {
        avlong = 0.5*(endlong + startlong);
        latmin = fmin(endlat, startlat);
        latmax = fmax(endlat, startlat);
        longmin = avlong - 0.5*latdif;
        longmax = avlong + 0.5*latdif;
    }
    else    {
        longmin = fmin(endlong, startlong);
        longmax = fmax(endlong, startlong);
        latmin = fmin(endlat, startlat);
        latmax = fmax(endlat, startlat);
    }


    printf("New values\n");
    printf("Minimum longitude value: %lf\n", longmin);
    printf("Maximum longitude value: %lf\n", longmax);
    printf("Minimum latitude value: %lf\n", latmin);
    printf("Maximum latitude value: %lf\n", latmax);
    printf("\n");

    for(i = 0; i < length; i++) {
        if(x[i] > longmax || x[i] < longmin || y[i] > latmax || y[i] < latmin) {
            newlength--;
            x[i] = 1000.0;
        }
    }

    xt = (double*) malloc(sizeof(double)*newlength);
    yt = (double*) malloc(sizeof(double)*newlength);

    s = 0;
    for(i = 0; i < length; i++) {
        if(x[i] < 1000.0){
            xt[s] = x[i];
            yt[s] = y[i];
            s++;
        }
    }

    free(x);
    free(y);

    length = newlength;

    x = (double*) malloc(sizeof(double)*length);
    y = (double*) malloc(sizeof(double)*length);

    for (i = 0; i < length; i++) {
        x[i] = xt[i];
        y[i] = yt[i];
    }
    free(xt);
    free(yt);


    /*
    **  Print out the (long,lat) - coordinates in the limited area
    */
    for (i = 0; i < length; i++) {
        fprintf(out6, "%lf %lf\n", x[i], y[i]);
    }
    fclose(out6);

    // Print out the given start and endpoint coordinates
    fprintf(out9,"%lf ", startlong);
    fprintf(out9,"%lf\n", startlat);
    fprintf(out9,"%lf ", endlong);
    fprintf(out9,"%lf", endlat);
    fclose(out9);


    /*
    **  Transform the maximum points from [deg] to [m] to get the span of the area
    **  Ask for grid size in meters
    **  Calculate the number of gridpoints:
    */

    /*
    **  Check if the area covers the given points
    */
    printf("longmin %lf\n", longmin);
    printf("longmax %lf\n", longmax);
    printf("startlong %lf\n", startlong);
    printf("endlong %lf\n", endlong);
    printf("\n");
    printf("latmin %lf\n", latmin);
    printf("latmax %lf\n", latmax);
    printf("startlong %lf\n", startlat);
    printf("endlong %lf\n", endlat);

    degrad = 1.0/180.0;
    xmax = pi*R*degrad*(longmax - longmin);
    ymax = pi*R*degrad*(latmax - latmin);

    while (choise != 'y'){

        printf("\n");
        printf("Enter the prefered distance (in meters) between grid points:  ");
        scanf("%lf", &gridsize);
        printf("\n");

        N = ((int)(ymax/gridsize));
        printf("The given distance gives a %i x %i grid\n", N, N);
        printf("Press 'y' to continue, or press 'n' to change the given distance:  ");
        scanf(" %c", &choise);
        printf("\n");
    }

    //Create plotscript for the path over the reduced dataset, need N
    fprintf(gpl2, "cd '%s'\n",dir);
    fprintf(gpl2, "set term png enhanced\n");
    fprintf(gpl2, "unset key\n");
    fprintf(gpl2, "set xrange [0:%i]\n",N);
    fprintf(gpl2, "set yrange [0:%i]\n",N);
    fprintf(gpl2, "set xlabel \"i\"\n");
    fprintf(gpl2, "set ylabel \"j\" rotate by 0\n");
    fprintf(gpl2, "set out 'partial_data.png'\n");
    fprintf(gpl2, "plot 'coord_trans.dat' using 1:2 with dots, 'endpoints.dat' using 1:2:(0.9) with circles lc rgb \"blue\" fs transparent solid, 'minimumvalues.dat' using 1:2 with points pt 7 ps 1 lc rgb \"black\"\n");
    fprintf(gpl2, "exit");
    fclose(gpl2);

    /*
    **  Allocate memory for arrays, which size is given by N
    */

    grid2d = (double**) malloc(sizeof(double*)*(N+1));
    for (i = 0; i < N+1; i++) {
        grid2d[i] = (double*) malloc(sizeof(double)*(N+1));
    }
    // Initializing grid
    for (i = 0; i < N+1; i++) {
        for (j = 0; j < N+1; j++) {
            grid2d[i][j] = 0.0;
        }
    }


    /*
    ** Transformation
    */

    for(i = 0; i < length; i++) {
        x[i] = (N-1)*(( pi*R*degrad*(x[i] - longmin) )/xmax);
        y[i] = (N-1)*(( pi*R*degrad*(y[i] - latmin) )/ymax);
    }

    // Transform the endpoints seperately and write to file (out7)
    fprintf(out7, "%i ", (int)((N-1)*(( pi*R*degrad*(startlong - longmin) )/xmax)));
    fprintf(out7, "%i\n", (int)((N-1)*(( pi*R*degrad*(startlat - latmin) )/ymax)));
    fprintf(out7, "%i ", (int)((N-1)*(( pi*R*degrad*(endlong - longmin) )/xmax)));
    fprintf(out7, "%i\n", (int)((N-1)*(( pi*R*degrad*(endlat - latmin) )/ymax)));


    // Count with "fuzzy - weights"

    for(i = 0; i < length; i++) {

        // Find the lower left corner
        xint = (int)(x[i]);
        yint = (int)(y[i]);

        // Find the distances to each corner of the surrounding square

        r1 = sqrtf( (x[i]- xint)*(x[i]- xint) + (y[i] - yint)*(y[i] - yint) );
        r2 = sqrtf( (x[i]- xint)*(x[i]- xint) + (y[i] - (yint+1))*(y[i] - (yint+1)) );
        r3 = sqrtf( (x[i]- (xint+1))*(x[i]- (xint+1)) + (y[i] - (yint+1))*(y[i] - (yint+1)) );
        r4 = sqrtf( (x[i]- (xint+1))*(x[i]- (xint+1)) + (y[i] - yint)*(y[i] - yint) );

        rtot = r1 + r2 + r3 + r4;

        w1 = (rtot - r1)/(3*rtot);
        w2 = (rtot - r2)/(3*rtot);
        w3 = (rtot - r3)/(3*rtot);
        w4 = (rtot - r4)/(3*rtot);

        grid2d[xint][yint] += w1;
        grid2d[xint][yint+1] += w2;
        grid2d[xint+1][yint+1] += w3;
        grid2d[xint+1][yint] += w4;

         fprintf(out3,"%lf %lf \n", x[i], y[i]);
    }

    for (i = 0; i < N+1; i++) {
        for (j = 0; j < N+1; j++) {

            // Print all grid values
            fprintf(out1,"%i %i %lf\n", i, j, grid2d[i][j]);

            // Print all non-zero grid values
            if(grid2d[i][j] > 0.0000001)
                fprintf(out2,"%i %i %lf\n", i, j, grid2d[i][j]);
        }
    }

    /*
    ** Write out relevant physical information to file
    */

    xmin = 0;
    ymin = 0;


    fprintf(out4, "Physical data: \n");
    fprintf(out4, "\n");
    fprintf(out4, "AIS - data [deg]\n");
    fprintf(out4, " Min longitude: %lf\n", longmin);
    fprintf(out4, " Min latide: %lf\n", latmin);
    fprintf(out4, " Max longitude: %lf\n", longmax);
    fprintf(out4, " Max latitude: %lf\n", latmax);
    fprintf(out4, "\n");
    fprintf(out4, "Transformed data [m]\n");
    fprintf(out4, " Min x-value: %lf\n", xmin);
    fprintf(out4, " Min y-value: %lf\n", ymin);
    fprintf(out4, " Max x-value: %lf\n", xmax);
    fprintf(out4, " Max y-value: %lf\n", ymax);
    fprintf(out4, "\n");
    fprintf(out4, "Grid data\n");
    fprintf(out4, " Grid size: %i\n", N);
    fprintf(out4, " Lengthsteps in x-direction: %lf\n", xmax/N);
    fprintf(out4, " Lengthsteps in y-direction: %lf\n", ymax/N);

    /*
    ** Write a reduced physical data file containg the boundary values for longitude and latitude
    ** This file, out5 - "phys_reduced.dat" is for further use in "transback.c"
    */

    fprintf(out5, "%lf\n", longmin);
    fprintf(out5, "%lf\n", latmin);
    fprintf(out5, "%lf\n", longmax);
    fprintf(out5, "%lf\n", latmax);

    fclose(out1);
    fclose(out2);
    fclose(out3);
    fclose(out4);
    fclose(out5);
    fclose(out7);


    pathscape(dir, N-1, alpha);
    repl(dir, N);
    transplot(dir, N);

    sprintf(gnu,"%s %s","gnuplot", gpl2n);
    system(gnu);


    return 0;
}
