#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include <math.h>

/*
**	pathscapeorg.c	- Implementation of the pathscape algorithm, used for predicting single routes
**		
**		pathscape(char[] dir, int N, double alpha) - No return variables, all produced data is printed
**													 to datafiles in the directory given by "dir"
**
*/


void fillGrid();
void update();

FILE *grid;

int N, i, j, k, dnew, dold;

// Parameters for analysis
double infty = 100.0;
double inftybound = 10000;
double eps = 0.00000001;
double pot;					// set to alpha

bool condition;


char dir[150];
double **dens;

// V
double ***v;
double ***tv;

void pathscape (char directory[],int n, double alpha) {

    sprintf(dir, directory);
    pot = alpha;
    N = n;

    /*
    ** Opening files
    */
    //Names
    char gridn[200], in1n[200], in2n[200], out1n[200], out2n[200], gpl1n[200], gpl2n[200];
    sprintf(gridn, "%s%s", dir, "\\gridvalues.dat");
    sprintf(in1n, "%s%s", dir, "\\endpoints.dat");
    sprintf(out1n, "%s%s", dir, "\\vij.dat");
    sprintf(gpl1n, "%s%s", dir, "\\plot.txt");
    sprintf(gpl2n, "%s%s", dir, "\\plot_no.txt");



    // Open
    grid = fopen(gridn,"r");
    FILE *in = fopen(in1n, "r");
    FILE *out1 = fopen(out1n, "w");
    FILE *gpl1 = fopen(gpl1n,"w");
    FILE *gpl2 = fopen(gpl2n,"w");

    // Allocate arrays
    dens = (double**) malloc(sizeof(double*)*(N+1));
    for(i = 0; i < N+1; i++) {
        dens[i] = (double*) malloc(sizeof(double)*(N+1));
    }

    v = (double***) malloc(sizeof(double**)*2);
    tv = (double***) malloc(sizeof(double**)*2);
    for (i = 0; i < 2; i++) {
        v[i] = (double**) malloc(sizeof(double*)*(N+1));
        tv[i] = (double**) malloc(sizeof(double*)*(N+1));
    }
    for (i = 0; i < 2; i++) {
        for (j = 0; j < N+1; j++) {
            v[i][j] = (double*) malloc(sizeof(double)*(N+1));
            tv[i][j] = (double*) malloc(sizeof(double)*(N+1));
        }
    }

    fillGrid();
    fclose(grid);

    int sx, sy;
    // Read the given endpoints from file "endpoints.dat"

        fscanf(in,"%i\n",  &sx);
        fscanf(in,"%i\n",  &sy);

        if  (sx == N-1)
            sx = N;
        if  (sy == N-1)
            sy = N;

        v[0][sx][sy] = 0.0;
        tv[0][sx][sy] = 0.0;

        fscanf(in,"%i\n",  &sx);
        fscanf(in,"%i\n",  &sy);

        if  (sx == N-1)
            sx = N;
        if  (sy == N-1)
            sy = N;
        v[1][sx][sy]= 0.0;
        tv[1][sx][sy] = 0.0;

        fclose(in);

    condition = false;

    int itcount = 0;
    while(condition != true) {
        dnew = 0.0;
        itcount++;
        update();

        for (i = 1; i < N; i++) {
            for (j = 1; j < N; j++) {
                dnew = dnew + v[0][i][j] + v[1][i][j];
            }
        }

        if(fabs(dnew - dold) <= eps)
            condition = true;

        dold = dnew;

        if(itcount % 100 == 0)
            printf("Iterations: %i \n", itcount);
    }
    printf("Iterations: %i\n", itcount);

    printf("\n");
    printf("\n");
    printf("\n");
    printf("Writing to file");
    printf("\n");


    for (i = 0; i < N+1; i++) {
        for (j = 0; j < N+1; j++) {
            fprintf(out1,"%i %i %lf\n", i, j, v[0][i][j] + v[1][i][j]);
        }
    }
    fclose(out1);

    /*
    **  Create plotscript and plot
    */

    fprintf(gpl1,"cd '%s'\n",dir);
    fprintf(gpl1,"set term pdf\n");
    fprintf(gpl1,"set out 'pathscape.pdf'\n");
    fprintf(gpl1,"set view map\n");
    fprintf(gpl1,"set palette gray\n");
    fprintf(gpl1,"splot 'vij.dat' using 1:2:3 with image\n");
    fprintf(gpl1,"exit\n");
    fclose(gpl1);

    char plot[200];
    sprintf(plot,"%s %s","gnuplot",gpl1n);
    system(plot);

    fprintf(gpl2,"cd '%s'\n",dir);
    fprintf(gpl2,"set term pdf\n");
    fprintf(gpl2,"set out 'pathscape_modified.pdf'\n");
    fprintf(gpl2,"set view map\n");
    fprintf(gpl2,"set palette gray\n");
    fprintf(gpl2,"splot 'vij_modified.dat' using 1:2:3 with image\n");
    fprintf(gpl2,"exit\n");
    fclose(gpl2);

    sprintf(plot,"%s %s", "gnuplot", gpl2n);
    system(plot);
}

void fillGrid(){

    double read;
    int toss;

    // Read in the # ships corresponding to each grid point and inverse the value.
    for (i = 0; i < N+1; i++) {
        for(j = 0; j < N+1; j++) {
            fscanf(grid, "%i %i %lf\n", &toss, &toss, &read);
            if(read > 0.000001)
                dens[i][j] = pow(1.0/read, pot);
            else
                dens[i][j] = pow(infty,pot);
        }
    }

    for (i = 0; i < 2; i++) {
        for (j = 0; j < N+1; j++) {
            for (k = 0; k < N+1; k++) {

                v[i][j][k] = 0.0;

                if  (j == 0 || j == N || k == 0  || k == N )
                    v[i][j][k] = inftybound;

                tv[i][j][k] = v[i][j][k];
            }
        }
    }

}

void update() {

    double minv, up1, up2, up3, up4, up5, up6, up7, up8;

    for (i = 0; i < 2; i++) {
        for (j = 1; j < N; j++) {
            for (k = 1; k < N; k++) {

                tv[i][j][k] = v[i][j][k];
            }
        }
    }

    for (i = 0; i < 2; i++) {
        for (j = 1; j < N; j++) {
            for (k = 1; k < N; k++) {

                up1 = (dens[j][k+1] + dens[j][k])/2.0 + tv[i][j][k+1];
                up2 = (dens[j][k-1] + dens[j][k])/2.0 + tv[i][j][k-1];
                up3 = (dens[j+1][k] + dens[j][k])/2.0 + tv[i][j+1][k];
                up4 = (dens[j-1][k] + dens[j][k])/2.0 + tv[i][j-1][k];
                up5 = sqrtf(2.0)*(dens[j+1][k+1] + dens[j][k])/2.0 + tv[i][j+1][k+1];
                up6 = sqrtf(2.0)*(dens[j+1][k-1] + dens[j][k])/2.0 + tv[i][j+1][k-1];
                up7 = sqrtf(2.0)*(dens[j-1][k+1] + dens[j][k])/2.0 + tv[i][j-1][k+1];
                up8 = sqrtf(2.0)*(dens[j-1][k-1] + dens[j][k])/2.0 + tv[i][j-1][k-1];

                minv = up1;
                if(up2 <= minv)
                    minv = up2;
                if(up3 <= minv)
                    minv = up3;
                if(up4 <= minv)
                    minv = up4;
                if(up5 <= minv)
                    minv = up5;
                if(up6 <= minv)
                    minv = up6;
                if(up7 <= minv)
                    minv = up7;
                if(up8 <= minv)
                    minv = up8;

                v[i][j][k] = minv;
            }
        }
    }
}
