#include <stdio.h>
#include <stdlib.h>
#include <math.h>




int N;
int R = 6371000;
double pi = 3.14159265359;

double trans(double a, int b);

double *x, *y;

double degrad, xmax, ymax;
double longmin, longmax, latmin, latmax, toss;
int nPoints, xin, yin;


void transplot (char directory[], int n) {

    // Directory and file names
    char dir[150];
    sprintf(dir, directory);
    N = n;

    char in1n[200], in2n[200], in3n[200], physn[200], outn[200], gpln[200], plot[200];

    sprintf(in1n, "%s%s", dir, "\\minimumvalues.dat");
    sprintf(in2n, "%s%s", dir, "\\number_val.dat");
    sprintf(physn, "%s%s", dir, "\\phys_reduced.dat");
    sprintf(outn, "%s%s", dir, "\\btrans.dat");
    sprintf(gpln, "%s%s", dir, "\\plot_transformed_data.txt");
    sprintf(plot, "%s %s", "gnuplot", gpln);

    FILE *in1 = fopen(in1n,"r");
    FILE *in2 = fopen(in2n,"r");
    FILE *phys = fopen(physn,"r");
    FILE *out = fopen(outn,"w");
    FILE *gpl = fopen(gpln,"w");

    int i, j;
    double val, d;

    fscanf(phys,"%lf\n", &longmin);
    fscanf(phys,"%lf\n", &latmin);
    fscanf(phys,"%lf\n", &longmax);
    fscanf(phys,"%lf\n", &latmax);

    printf("longmin: %lf\n", longmin);
    printf("latmin: %lf\n", latmin);
    printf("longmax: %lf\n", longmax);
    printf("latmax: %lf\n", latmax);

    fclose(phys);

    fscanf(in2,"%i\n", &nPoints); // Read in number of points present in the optimal path

    nPoints = nPoints;

    //Allocate memory for data
    x = (double*) malloc(sizeof(double)*nPoints);
    y = (double*) malloc(sizeof(double)*nPoints);


    for(i = 0; i < nPoints; i++){
        fscanf(in1, "%i %i %lf\n", &xin, &yin, &toss);
        x[i] = (double)(xin);
        y[i] = (double)(yin);
    }

    fclose(in1);
    fclose(in2);


    //transformation

    degrad = 1.0/180.0;

    xmax = pi*R*degrad*(longmax - longmin);
    ymax = pi*R*degrad*(latmax - latmin);

    for (i = 0; i < nPoints; i++) {
        fprintf(out, "%lf %lf\n", trans(x[i],0), trans(y[i],1));
    }
    fclose(out);


    /*
    **  Create gnuplot script and plot data
    */

    fprintf(gpl,"cd '%s'\n",dir);
    fprintf(gpl,"set term png\n");
    fprintf(gpl,"set out 'optimalpath_over_dataset.png'\n");
    fprintf(gpl,"unset key\n");
    fprintf(gpl,"plot 'btrans.dat' using 1:2, 'coords_reduced.dat' using 1:2 with dots\n");
    fprintf(gpl, "set out 'optimalpath_over_realpath.png'\n");
    fprintf(gpl, "plot 'btrans.dat' using 1:2, 'route_coordinates.dat' using 1:2 with lines\n");
    fprintf(gpl, "set out 'optimalpath_over_realpath_square.png'\n");
    fprintf(gpl, "set size square\n");
    fprintf(gpl, "plot 'btrans.dat' using 1:2 with points pt 7 ps 1 lc rgb \"black\", 'route_coordinates.dat' using 1:2 with lines\n");
    fprintf(gpl,"exit\n");
    fclose(gpl);

    system(plot);

    return 0;
}


double trans(double a, int b) {

    double c;

    if(b == 0){
        c = ((a*xmax)/((N-1)*(pi*R*degrad))) + longmin;
    }
    else if(b == 1){
        c = ((a*ymax)/((N-1)*(pi*R*degrad))) + latmin;
    }

    return c;
}
